#!/bin/bash

source $(dirname $0)/config.sh $1
bindir=~/work/fibre/apps

ln -sf $currdir/ulib_webserver.sh $bindir/ewebserver
ln -sf fwebserver $bindir/swebserver

function run() {
	exp=$1
  type=$2
  rep=$3
	threads=$4
	conns=$5
	aux=$6

	unset FibreDefaultPollers
	case $exp in
		exp1)
			botcpu=$(expr $top_server_core - $max_server_threads)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$reg_client_cores
			client_threads=$reg_client_threads
			export FibreDefaultPollers=$(expr 1 + $(expr $threads / 4))
			case "$type" in
				scope) return;;
			esac;;
		exp5)
			botcpu=$(expr $top_server_core - $max_server_threads)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$reg_client_cores
			client_threads=$reg_client_threads
			export FibreDefaultPollers=$(expr 1 + $(expr $threads / $aux))
			case "$type" in
				scope|upp|pthreads|events|go) return;;
			esac;;
		exp2)
			botcpu=$(expr $top_server_core - 16)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$max_client_cores
			client_threads=$max_client_threads
			case "$type" in
				scope|upp) return;;
			esac;;
		exp3)
			botcpu=$(expr $top_server_core - 16)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$max_client_cores
			client_threads=$max_client_threads
			case "$type" in
				scope|upp|pthreads) return;;
			esac;;
		exp4)
			botcpu=$(expr $top_server_core - $max_server_threads)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$reg_client_cores
			client_threads=$reg_client_threads
			case "$type" in
				go) [ $threads -gt 16 ] && return;; # Go cannot handle more than 16 anyway
				upp) return;;
				scope)
					[ $threads -ge 4 ] && server_config="-e 4 -t $(expr $threads / 4)";;
			esac;;
		exp6)
			botcpu=$(expr $top_server_core - $max_server_threads)
			topcpu=$(expr $botcpu + $threads - 1)
			server_config="-t $threads"
			client_cores=$reg_client_cores
			client_threads=$reg_client_threads
			[ $threads -ge $aux ] && server_config="-e $aux -t $(expr $threads / $aux)"
			case "$type" in
        fibre|upp|pthreads|events|go) return;;
			esac;;
		*) return;;
	esac

	# handle a special case
	[ "$type" = "pthreads" ] && duration=180 || duration=60

	# set up executable and file names
	server_prog=$(echo $type|cut -c1)webserver
	outdir=$basedir/webserver/$exp
	mkdir -p $outdir
	file=$outdir/$type-$threads-$conns.$rep
	[ "$aux" ] && file=$file.$aux
	[ -f $file.out ] && return;
	[ -L $file.out ] && return;

	# wait, if previous crashes have left timed-wait sockets
	while ss state time-wait|fgrep -q 127.0.0.1:8800; do sleep 1; done

	# report run
	report_run $file

	# start server
	taskset -c $botcpu-$topcpu $bindir/$server_prog $server_config &
	# wait a moment...
	sleep 1

	if [ "$exp" = "exp6" ]; then
		wtime=1
		while [ $(ss state listening|fgrep 0.0.0.0:8800|wc -l) -lt $aux ]; do
			killall $server_prog
			sleep $wtime
			taskset -c $botcpu-$topcpu $bindir/$server_prog $server_config &
			sleep $wtime
			wtime=$(expr $wtime \* 2)
		done
	fi

	# start clients, warmup then run
	case $exp in
		exp1|exp2|exp5)
			taskset -c $client_cores timeout  60 wrk --timeout 1s --latency -d 10s -t $client_threads -c $conns http://127.0.0.1:8800/plaintext # warm up
			sleep 1
			to=$(expr ${duration} \* 2)
			taskset -c $client_cores timeout $to wrk --timeout 1s --latency -d ${duration}s -t $client_threads -c $conns http://127.0.0.1:8800/plaintext | tee $file.out
			;;
		exp3)
			taskset -c $client_cores $(pwd)/multi_wrk.sh 20 $conns $client_threads 10s 180 # warm up
			sleep 1
			to=$(expr ${duration} \* 6)
			taskset -c $client_cores $(pwd)/multi_wrk.sh 20 $conns $client_threads ${duration}s $to | tee $file.out
			;;
		exp4|exp6)
			config="-t $(expr $threads \* 2) -c $(expr $threads \* 200) -n $(expr $threads \* 200000)"
			taskset -c $client_cores timeout 90 weighttp $config http://localhost:8800/plaintext
			sleep 1
			taskset -c $client_cores timeout 90 weighttp $config http://localhost:8800/plaintext | tee $file.out
			;;
	esac

	# killall does not work with ULib's server
	[ "$type" = "events" ] && killname=userver_tcp || killname=$server_prog
	# kill nicely once, then use -9
	kill_it $killname
	unset FibreDefaultPollers
}

for ((r=1;$r<=$reps;r+=1)); do
	for t in $webserver_types; do
		for p in $webserver_parlevels; do
			run exp1 $t $r $p 15000
			run exp4 $t $r $p 0
		done
		for x in 1 2 4 8 16 32; do
			run exp5 $t $r 32 15000 $x
			run exp6 $t $r 32 0 $x
		done
		for c in $webserver_conns; do
			run exp2 $t $r 16 $c
		done
		for c in $webserver_bigconns; do
			run exp3 $t $r 16 $c
		done
	done # variants
done # reps

exit 0
