#!/bin/bash
[ $(id -u) -ne 0 ] && exec sudo $0 $*

fgrep -q "UserTasksMax=infinity" /etc/systemd/logind.conf || {
	echo UserTasksMax=infinity >> /etc/systemd/logind.conf
}

for f in /sys/fs/cgroup/pids/user.slice/pids.max \
         /sys/fs/cgroup/pids/user.slice/user-*.slice/pids.max; do
	fgrep -q max $f || {
	  echo "UserTasksMax=infinity has not taken effect yet - reboot?"
	  exit 0
	}
done

if [ "$1" = "reset" ]; then
	sysctl -w net.core.somaxconn=128
	sysctl -w net.ipv4.tcp_max_syn_backlog=256
	sysctl -w net.ipv4.tcp_tw_reuse=0
	sysctl -w net.ipv4.ip_local_port_range="32768 60999"
	sysctl -w vm.max_map_count=65530
	cpupower frequency-set -g powersave
	for ((i=2;i<=32;i++)); do ifconfig lo:$i down 2>/dev/null; done
else
	sysctl -w net.core.somaxconn=1048576
	sysctl -w net.ipv4.tcp_max_syn_backlog=262144
	sysctl -w net.ipv4.tcp_tw_reuse=1
	sysctl -w net.ipv4.ip_local_port_range="1024 65535"
	sysctl -w vm.max_map_count=2097152
	cpupower frequency-set -g performance
	for ((i=2;i<=32;i++)); do ifconfig lo:$i inet 127.0.0.$i netmask 255.255.255.0 up; done
fi


# various debug-related settings
echo 0 > /proc/sys/kernel/yama/ptrace_scope         # gdb attach
echo -1 > /proc/sys/kernel/perf_event_paranoid      # perf anything
echo 0 > /proc/sys/kernel/kptr_restrict             # perf tracing
echo 0 > /proc/sys/kernel/nmi_watchdog              # perf cache tracing

# perf syscall tracing
mount -t debugfs debugfs /sys/kernel/debug 2>/dev/null
mount -o remount,mode=755 /sys/kernel/debug
mount -t tracefs tracefs /sys/kernel/debug/tracing 2>/dev/null
mount -o remount,mode=755 /sys/kernel/debug/tracing

# add to /etc/security/limits.conf or store in /etc/security/limits.d/$USER.conf
echo "\
mkarsten hard nofile 1048576
mkarsten soft nofile 1048576" > /etc/security/limits.d/mkarsten.conf

# add to /etc/sudoers or store in /etc/sudoers.d/$USER
echo "mkarsten ALL = (ALL) NOPASSWD: ALL" > /etc/sudoers.d/mkarsten

# EXIT HERE, below are just notes
exit 0

# monitor file descriptors
cat /proc/sys/fs/file-nr

# monitor threads
ps -eLf|fgrep memcached|wc

# see "Performance Scalability of a Multi-Core Web Server", Nov 2007
# Bryan Veal and Annie Foong, Intel Corporation, Page 4/10
sysctl -w fs.file-max=5000000
sysctl -w fs.nr_open=500000
sysctl -w net.core.netdev_max_backlog=1000000
sysctl -w net.core.optmem_max=10000000
sysctl -w net.core.rmem_default=10000000
sysctl -w net.core.rmem_max=10000000
sysctl -w net.core.wmem_default=10000000
sysctl -w net.core.wmem_max=10000000
sysctl -w net.ipv4.conf.all.rp_filter=1
sysctl -w net.ipv4.conf.default.rp_filter=1
sysctl -w net.ipv4.tcp_congestion_control=bic
sysctl -w net.ipv4.tcp_ecn=0
sysctl -w net.ipv4.tcp_max_syn_backlog=12000
sysctl -w net.ipv4.tcp_max_tw_buckets=2000000
sysctl -w net.ipv4.tcp_mem="30000000 30000000 30000000"
sysctl -w net.ipv4.tcp_rmem="30000000 30000000 30000000"
sysctl -w net.ipv4.tcp_wmem="30000000 30000000 30000000"
sysctl -w net.ipv4.tcp_sack=1
sysctl -w net.ipv4.tcp_syncookies=0
sysctl -w net.ipv4.tcp_timestamps=1
